/**
 * @file
 * Provides SlickLightbox loader.
 */

(function ($, Drupal, drupalSettings, _db) {

  'use strict';

  var _id = 'slick';
  var _name = _id + '-lightbox';
  var _idOnce = _name;
  var _galleryId = _name + '-gallery';
  var _mounted = _galleryId + '--on';
  var _selector = '[data-' + _galleryId + ']';
  var _base = _name + '-slick';
  var _baseSel = '.' + _base;
  var _nick = 'slbox';
  var _unload = false;
  var _isPlaying = 'is-playing';
  var _isPaused = 'is-paused';

  /**
   * SlickLightbox utility functions.
   *
   * @param {HTMLElement} elm
   *   The SlickLightbox gallery HTML element.
   */
  function process(elm) {
    var sbox;
    var $slider = $('> .slick__slider', elm);
    var $container = $slider.length ? $slider : $(elm);
    var boxSettings = drupalSettings.slickLightbox.lightbox || {};
    var slickSettings = drupalSettings.slickLightbox.slick || {};

    boxSettings.itemSelector = boxSettings.itemSelector + ',[data-' + _name + '-trigger]';
    var itemSelector = boxSettings.itemSelector;
    var $triggers = $(itemSelector, elm);

    function getIndex(item) {
      var i = 0;

      // Only refetch dynamic items with AJAX loaded contents such as IO, VIS.
      if (_unload) {
        $triggers = $container.find(itemSelector);
        _unload = false;
      }

      $triggers.each(function (idx, elm) {
        if (elm === item) {
          i = idx;
          return false;
        }
      });
      return i;
    }

    // Initializes slick with video supports.
    function initSlick(modalElement) {
      var media;
      var $box;
      var $media;
      var $player;
      var $video;
      var $slide;
      var $slick = $(_baseSel, modalElement);
      var $slides = $slick.children();
      var $instance;

      /**
       * Trigger the media close.
       */
      function closeOut() {
        var $playing = $slick.find('.' + _isPlaying);
        $video = $slick.find('video');

        // Clean up any pause marker at slider container.
        $slick.removeClass(_isPaused);

        if ($playing.length) {
          $playing.removeClass(_isPlaying).find('.media__icon--close').click();
        }

        if ($video.length) {
          $video[0].pause();
        }
      }

      /**
       * Trigger pause on slick instance when playing a video.
       */
      function pause() {
        $slick.addClass(_isPaused).slick('slickPause');
      }

      /**
       * Resize the responsive image.
       */
      function resizeImage() {
        var t = $(this);
        var w = t.width();
        var p = t.closest('.media-wrapper--inline');
        var c = p.next(_baseSel + '-caption');
        var ph = p.parent().height();

        if (c.length) {
          ph = ph - (c.height() + parseInt(c.css('margin-top').replace('px', '')));
        }

        t.css('max-height', ph);
        p.css({width: w});
      }

      /**
       * Build out the media player.
       *
       * @param {int} i
       *   The index of the current element.
       * @param {HTMLElement} box
       *   The gallery item HTML element which triggers the lightbox.
       */
      function buildOutMedia(i, box) {
        var html;
        $box = $(box);
        $slide = $($slides[i]);
        $player = $('.media--player', $slide);
        $video = $('video', $slide);
        media = $box.data('media');

        // Hook into native to fix potential lazy load issues.
        $('img', $slide).attr('loading', 'lazy');

        if (media) {
          $slide.addClass('slick-slide--' + media.type);
          // @todo remove bRich.
          var $html = media.html || $box.data('bRich') || false;
          if (media.type === 'rich' && $html && !$video.length) {
            try {
              html = 'html' in media ? $html : JSON.parse($html);
              $media = Drupal.theme('slickLightbox', {html: html, media: media});

              $slide.find(_baseSel + '-img').replaceWith($media);

              var $img = $slide.find('img');
              if ($img.length) {
                $img.addClass(_base + '-img');
                if ($slide.find(_baseSel + '-caption').length) {
                  window.setTimeout(function () {
                    $img.each(function () {
                      if (this.complete) {
                        resizeImage.call(this);
                      }
                      else {
                        $(this).one('load', resizeImage);
                      }
                    });
                  }, 10);
                }
              }
            }
            catch (e) {
              // Ignore.
            }

            Drupal.attachBehaviors($video[0]);
          }
          else {
            if ($box.data('boxUrl') && !$player.length) {
              $media = Drupal.theme('blazyMedia', {el: box});

              $slide.find(_baseSel + '-img').replaceWith($media);

              Drupal.attachBehaviors($player[0]);
            }
          }
        }
      }

      // Initializes slick.
      if (!$slick.hasClass('slick-initialized')) {
        $triggers.each(buildOutMedia);

        $instance = $slick.slick(slickSettings);

        $slick.on('afterChange.' + _nick, closeOut);
        $slick.on('click.' + _nick, '.media__icon--close', closeOut);
        $slick.on('click.' + _nick, '.media__icon--play', pause);

        // Fix for broken display when only has 1 slide, like usual.
        if ($slides.length === 1) {
          var slick = $slick.slick('getSlick');
          $slick.addClass(_base + '--unslick');
          slick.$slideTrack.css({left: '', transform: ''});
        }
      }

      return $instance;
    }

    var options = {
      // Prevents clicking a video player button from closing the lightbox.
      // @todo re-enable when the library provides a fix for this.
      closeOnBackdropClick: false,
      itemSelector: itemSelector,
      caption: function (target, info) {
        var $caption = $(target).next('.litebox-caption');
        return $caption.length ? $caption.html() : '';
      },
      src: function (target) {
        var $target = $(target);
        return $target.data('boxUrl') || $target.attr('href');
      },
      slick: initSlick
    };

    var events = {
      'show.slickLightbox': function () {
        // Prevents media player with aspect ratio from being squeezed.
        $('.slick-slide--video .' + _base + '-item-inner').removeAttr('style');

        // Overrides closeOnBackdropClick as otherwise clicking video play
        // button closes the entire lightbox.
        // @todo remove when the library fixes this.
        $($container[0].slickLightbox.$modalElement).on('click.' + _nick, _baseSel + '-item', function (e) {
          if (e.target === this) {
            $('.' + _name + '-close').click();
          }
        });
      }
    };

    // Initializes slick lightbox.
    var boxOptions = boxSettings ? $.extend({}, options, boxSettings) : options;

    /**
     * Open the lightbox, and sets the correct delta.
     *
     * @todo remove when the library provides index argument to its initSlick().
     */
    function open() {
      var $this = $(this);
      var delta = 0;
      var found = false;
      var containers = boxSettings.itemContainers;

      // Delta is set per field which is always 0 within a view, fix it.
      if (containers) {
        var len = containers.length;

        for (var i = 0; i < len; i++) {
          var $item = $this.closest(containers[i]);
          if ($item.length) {
            delta = $item.index();
            found = true;
            break;
          }
        }
      }

      // Assumes anything else such as multi-entity blazy, or slick.
      if (!found) {
        delta = getIndex($this[0]);
      }

      sbox = $container[0].slickLightbox;
      if (sbox && sbox.slick) {
        sbox.slick.slick('slickGoTo', delta, true);
      }
    }

    // Init the lightbox.
    $container.slickLightbox(boxOptions).on(events);

    // @todo remove when the library provides index argument to its initSlick().
    $container.off('click.' + _nick).on('click.' + _nick, itemSelector, open);
    $(elm).addClass(_mounted);
  }

  /**
   * Theme function for a blazy PhotoSwipe (Remote|local) video.
   *
   * @param {Object} settings
   *   An object with the following keys:
   * @param {Array} settings.item
   *   The array of item properties containing: media and html.
   *
   * @return {HTMLElement}
   *   Returns a HTMLElement object.
   */
  Drupal.theme.slickLightbox = function (settings) {
    var media = settings.media;
    var html;

    html = '<div class="media-wrapper media-wrapper--inline" style="width:' + media.width + 'px">';
    html += settings.html;
    html += '</div>';

    return html;
  };

  /**
   * Attaches Slick Lightbox gallery behavior to HTML element.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.slickLightbox = {
    attach: function (context) {
      context = _db.context(context);
      _db.once(process, _idOnce, _selector + ':not(.' + _mounted + ')', context);
    },
    detach: function (context, settings, trigger) {
      _unload = trigger === 'unload';
      if (trigger === 'unload') {
        _db.once.removeSafely(_idOnce, _selector, context);
      }
    }
  };

})(jQuery, Drupal, drupalSettings, dBlazy);
